@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed.h,v 1.51 2005/01/30 15:20:35 chpe Exp $
+ *  $Id: ephy-embed.h,v 1.56 2005/07/29 20:12:55 chpe Exp $
  */
 
 #ifndef EPHY_EMBED_H
@@ -116,6 +116,10 @@ struct _EphyEmbedIface
 					 EphyEmbedEvent *event);
 	void	 (* favicon)		(EphyEmbed *embed,
 					 const char *location);
+	void	 (* feed_link)		(EphyEmbed *embed,
+					 const char *type,
+					 const char *title,
+					 const char *address);
 	void	 (* location)		(EphyEmbed *embed,
 					 const char *location);
 	void	 (* net_state)		(EphyEmbed *embed,
@@ -125,6 +129,8 @@ struct _EphyEmbedIface
 					 EphyEmbedEvent *event);
 	gboolean (* dom_mouse_down)	(EphyEmbed *embed,
 					 EphyEmbedEvent *event);
+	void     (* dom_content_loaded)	(EphyEmbed *embed,
+					 gpointer event);
 	void	 (* popup_blocked)	(EphyEmbed *embed,
 					 const char *address,
 					 const char *features);
@@ -140,6 +146,8 @@ struct _EphyEmbedIface
 					 EphyEmbedDocumentType type);
 	void	 (* new_window)		(EphyEmbed *embed,
 					 EphyEmbed *new_embed);
+	gboolean (* search_key_press)	(EphyEmbed *embed,
+					 GdkEventKey *event);
 
 	/* Methods  */
 	void		   (* load_url)			(EphyEmbed *embed,
@@ -176,12 +184,6 @@ struct _EphyEmbedIface
 	void		   (* set_zoom)			(EphyEmbed *embed,
 							 float zoom);
 	float		   (* get_zoom)			(EphyEmbed *embed);
-	void		   (* find_set_properties)	(EphyEmbed *embed,
-							 const char *search_string,
-							 gboolean case_sensitive,
-							 gboolean wrap_around);
-	gboolean	   (* find_next)		(EphyEmbed *embed,
-							 gboolean backwards);
 	char *		   (* get_encoding)		(EphyEmbed *embed);
 	gboolean	   (* has_automatic_encoding)	(EphyEmbed *embed);
 	void		   (* set_encoding)		(EphyEmbed *embed,
@@ -193,7 +195,6 @@ struct _EphyEmbedIface
 	void		   (* print_preview_navigate)	(EphyEmbed *embed,
 							 EphyEmbedPrintPreviewNavType type,
 							 int page);
-	void		   (* activate)			(EphyEmbed *embed);
 	gboolean	   (* has_modified_forms)	(EphyEmbed *embed);
 };
 
@@ -267,15 +268,6 @@ void		  ephy_embed_set_zoom			(EphyEmbed *embed,
 
 float		  ephy_embed_get_zoom			(EphyEmbed *embed);
 
-/* Find */
-void		  ephy_embed_find_set_properties	(EphyEmbed *embed,
-							 const char *search_string,
-							 gboolean case_sensitive,
-							 gboolean wrap_around);
-
-gboolean	  ephy_embed_find_next			(EphyEmbed *embed,
-							 gboolean backwards);
-
 /* Encoding */
 char		 *ephy_embed_get_encoding		(EphyEmbed *embed);
 
@@ -297,8 +289,6 @@ void		  ephy_embed_print_preview_navigate	(EphyEmbed *embed,
 							 int page);
 
 /* Misc. utility */
-void		  ephy_embed_activate			(EphyEmbed *embed);
-
 gboolean	  ephy_embed_has_modified_forms		(EphyEmbed *embed);
 
 G_END_DECLS
