@@ -1,7 +1,7 @@
 /*
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003-2004 Marco Pesenti Gritti
- *  Copyright (C) 2004 Christian Persch
+ *  Copyright (C) 2004, 2005 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,14 +17,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-favicon-cache.c,v 1.47 2005/07/08 21:15:19 chpe Exp $
+ *  $Id: ephy-favicon-cache.c,v 1.51 2005/08/08 20:18:52 chpe Exp $
  */
 
 #include "config.h"
 
 #include "ephy-favicon-cache.h"
 
-#include <libgnomevfs/gnome-vfs-ops.h>
 #include <string.h>
 #include <time.h>
 #include <sys/stat.h>
@@ -35,9 +34,12 @@
 #include "ephy-file-helpers.h"
 #include "ephy-node-common.h"
 #include "ephy-node.h"
+#include "ephy-debug.h"
+
 #include <glib/gstdio.h>
 #include <libgnomeui/libgnomeui.h>
-#include "ephy-debug.h"
+#include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-directory.h>
 
 #define EPHY_FAVICON_CACHE_XML_ROOT    (const xmlChar *)"ephy_favicons_cache"
 #define EPHY_FAVICON_CACHE_XML_VERSION (const xmlChar *)"1.1"
@@ -47,6 +49,12 @@
 /* how often to save the cache, in milliseconds */
 #define CACHE_SAVE_INTERVAL	10 * 60 * 1000 /* ms */
 
+/* how often to delete old pixbufs from the cache, in milliseconds */
+#define CACHE_CLEANUP_INTERVAL	5 * 60 * 1000 /* ms */
+
+/* how long to keep pixbufs in cache, in seconds. This should be longer than CACHE_CLEANUP_INTERVAL */
+#define PIXBUF_CACHE_KEEP_TIME	10 * 60 /* s */
+
 /* this is very generous, most files are 4k */
 #define EPHY_FAVICON_MAX_SIZE	64 * 1024 /* byte */
 
@@ -66,9 +74,22 @@ struct _EphyFaviconCachePrivate
 	GHashTable *icons_hash;
 	GHashTable *downloads_hash;
 	guint autosave_timeout;
+	guint cleanup_timeout;
 	guint dirty : 1;
+	guint64 requests;
+	guint64 cached;
 };
 
+typedef struct
+{
+	EphyNode *node;
+	time_t timestamp;
+	GdkPixbuf *pixbuf;
+	guint load_failed : 1;
+} PixbufCacheEntry;
+
+typedef gboolean (* FilterFunc) (EphyNode*, GDate *);
+
 enum
 {
 	CHANGED,
@@ -82,7 +103,7 @@ enum
 	EPHY_NODE_FAVICON_PROP_LAST_USED = 4,
 	EPHY_NODE_FAVICON_PROP_STATE	 = 5,
 	EPHY_NODE_FAVICON_PROP_CHECKOLD  = 6,
-	EPHY_NODE_FAVICON_PROP_CHECKED	 = 7
+	EPHY_NODE_FAVICON_PROP_CHECKED	 = 7,
 };
 
 enum
@@ -154,6 +175,17 @@ ephy_favicon_cache_new (void)
 	return EPHY_FAVICON_CACHE (g_object_new (EPHY_TYPE_FAVICON_CACHE, NULL));
 }
 
+static void
+pixbuf_cache_entry_free (PixbufCacheEntry *entry)
+{
+	if (entry->pixbuf != NULL)
+	{
+		g_object_unref (entry->pixbuf);
+	}
+
+	g_free (entry);
+}
+
 static gboolean
 icon_is_obsolete (EphyNode *node, GDate *now)
 {
@@ -175,14 +207,19 @@ icons_added_cb (EphyNode *node,
 		EphyNode *child,
 		EphyFaviconCache *eb)
 {
+	PixbufCacheEntry *entry = g_new0 (PixbufCacheEntry, 1);
+
+	entry->node = child;
+
 	g_hash_table_insert (eb->priv->icons_hash,
 			     (char *) ephy_node_get_property_string (child, EPHY_NODE_FAVICON_PROP_URL),
-			     child);
+			     entry);
 }
 
 static void
 icons_removed_cb (EphyNode *node,
 		  EphyNode *child,
+		  guint old_index,
 		  EphyFaviconCache *eb)
 {
 	g_hash_table_remove (eb->priv->icons_hash,
@@ -190,32 +227,32 @@ icons_removed_cb (EphyNode *node,
 }
 
 static void
-remove_obsolete_icons (EphyFaviconCache *eb)
+remove_obsolete_icons (EphyFaviconCache *cache,
+		       FilterFunc filter)
 {
+	EphyFaviconCachePrivate *priv = cache->priv;
 	GPtrArray *children;
 	int i;
-	GTime now;
 	GDate current_date;
 
-	now = time (NULL);
 	g_date_clear (&current_date, 1);
 	g_date_set_time (&current_date, time (NULL));
 
-	children = ephy_node_get_children (eb->priv->icons);
-	for (i = 0; i < children->len; i++)
+	children = ephy_node_get_children (priv->icons);
+	for (i = (int) children->len - 1; i >= 0; i--)
 	{
 		EphyNode *kid;
 
 		kid = g_ptr_array_index (children, i);
 
-		if (icon_is_obsolete (kid, &current_date))
+		if (!filter || filter (kid, &current_date))
 		{
 			const char *filename;
 			char *path;
 
 			filename = ephy_node_get_property_string
 				(kid, EPHY_NODE_FAVICON_PROP_FILENAME);
-			path = g_build_filename (eb->priv->directory,
+			path = g_build_filename (priv->directory,
 						 filename, NULL);
 			gnome_vfs_unlink (path);
 
@@ -265,6 +302,37 @@ periodic_save_cb (EphyFaviconCache *cache)
 	return TRUE;
 }
 
+static void
+cleanup_entry (char *key,
+	       PixbufCacheEntry *entry,
+	       time_t* now)
+{
+	if (entry->pixbuf != NULL &&
+	    *now - entry->timestamp > PIXBUF_CACHE_KEEP_TIME)
+	{
+		LOG ("Evicting pixbuf for \"%s\" from pixbuf cache", key);
+
+		g_object_unref (entry->pixbuf);
+		entry->pixbuf = NULL;
+		entry->load_failed = FALSE;
+		entry->timestamp = 0;
+	}
+}
+
+static gboolean
+periodic_cleanup_cb (EphyFaviconCache *cache)
+{
+	EphyFaviconCachePrivate *priv = cache->priv;
+	time_t now;
+
+	LOG ("Cleanup");
+
+	now = time (NULL);
+	g_hash_table_foreach (priv->icons_hash, (GHFunc) cleanup_entry, &now);
+
+	return TRUE;
+}
+
 static void
 ephy_favicon_cache_init (EphyFaviconCache *cache)
 {
@@ -292,13 +360,13 @@ ephy_favicon_cache_init (EphyFaviconCache *cache)
 		}
 	}
 
-	cache->priv->icons_hash = g_hash_table_new (g_str_hash,
-						    g_str_equal);
-	cache->priv->downloads_hash = g_hash_table_new_full (g_str_hash,
-							     g_str_equal,
-							     g_free,
-							     NULL);
-
+	/* The key is owned by the node */
+	priv->icons_hash = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
+						  (GDestroyNotify) pixbuf_cache_entry_free);
+	priv->downloads_hash = g_hash_table_new_full (g_str_hash,
+						      g_str_equal,
+						      g_free, NULL);
+	
 	/* Icons */
 	cache->priv->icons = ephy_node_new_with_id (db, ICONS_NODE_ID);
 	ephy_node_signal_connect_object (cache->priv->icons,
@@ -321,6 +389,9 @@ ephy_favicon_cache_init (EphyFaviconCache *cache)
 	priv->autosave_timeout = g_timeout_add (CACHE_SAVE_INTERVAL,
 						(GSourceFunc) periodic_save_cb,
 						cache);
+	priv->cleanup_timeout = g_timeout_add (CACHE_CLEANUP_INTERVAL,
+					       (GSourceFunc) periodic_cleanup_cb,
+					       cache);
 }
 
 static gboolean
@@ -345,6 +416,31 @@ kill_download (const char *key,
 	return TRUE;
 }
 
+static gboolean
+delete_file (const char *rel_path,
+	     GnomeVFSFileInfo *info,
+	     gboolean rec_will_loop,
+	     EphyFaviconCache *cache,
+	     gboolean *recurse)
+{
+	EphyFaviconCachePrivate *priv = cache->priv;
+	char *path;
+
+	*recurse = FALSE;
+
+	g_return_val_if_fail (info != NULL, TRUE);
+
+	if ((info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_TYPE) == 0 ||
+	    info->type != GNOME_VFS_FILE_TYPE_REGULAR) return TRUE;
+
+	path = g_build_filename (priv->directory, rel_path, NULL);
+	gnome_vfs_unlink (path);
+	g_free (path);
+
+	/* continue with the visit */
+	return TRUE;
+}
+
 static void
 ephy_favicon_cache_finalize (GObject *object)
 {
@@ -353,23 +449,32 @@ ephy_favicon_cache_finalize (GObject *object)
 
 	LOG ("EphyFaviconCache finalising");
 
-	g_hash_table_foreach_remove (cache->priv->downloads_hash,
+	g_hash_table_foreach_remove (priv->downloads_hash,
 				     (GHRFunc) kill_download, cache);
-	remove_obsolete_icons (cache);
 
 	if (priv->autosave_timeout != 0)
 	{
 		g_source_remove (priv->autosave_timeout);
 	}
 
+	if (priv->cleanup_timeout != 0)
+	{
+		g_source_remove (priv->cleanup_timeout);
+	}
+
+	remove_obsolete_icons (cache, icon_is_obsolete);
+
 	ephy_favicon_cache_save (cache);
 
-	g_free (cache->priv->xml_file);
-	g_free (cache->priv->directory);
-	g_hash_table_destroy (cache->priv->icons_hash);
-	g_hash_table_destroy (cache->priv->downloads_hash);
+	g_free (priv->xml_file);
+	g_free (priv->directory);
+	g_hash_table_destroy (priv->icons_hash);
+	g_hash_table_destroy (priv->downloads_hash);
 
-	g_object_unref (cache->priv->db);
+	g_object_unref (priv->db);
+
+	LOG ("Requests: cached %" G_GUINT64_FORMAT " / total %" G_GUINT64_FORMAT " = %.3f",
+	     priv->cached, priv->requests, ((double) priv->cached) / ((double) priv->requests));
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -456,12 +561,23 @@ ephy_favicon_cache_download (EphyFaviconCache *cache,
 	ephy_embed_persist_save (persist);
 }
 
+/**
+ * ephy_favicons_cache_get:
+ * @cache:
+ * @url: the URL of the icon to retrieve
+ * 
+ * Note: This will always return %NULL for non-http URLs.
+ * 
+ * Return value: the site icon at @url as a #GdkPixbuf, or %NULL if
+ * if could not be retrieved. Unref when you don't need it anymore.
+ */
 GdkPixbuf *
 ephy_favicon_cache_get (EphyFaviconCache *cache,
 			const char *url)
 {
 	EphyFaviconCachePrivate *priv = cache->priv;
-	GTime now;
+	time_t now;
+	PixbufCacheEntry *entry;
 	EphyNode *icon;
 	GValue value = { 0, };
 	char *pix_file;
@@ -470,11 +586,30 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 
 	if (url == NULL) return NULL;
 
+	if (!g_str_has_prefix (url, "http://")) return NULL;
+
+	priv->requests += 1;
+
 	now = time (NULL);
 
-	icon = g_hash_table_lookup (cache->priv->icons_hash, url);
+	entry = g_hash_table_lookup (priv->icons_hash, url);
 
-	if (!icon)
+	if (entry != NULL && entry->pixbuf != NULL)
+	{
+		LOG ("Pixbuf in cache for \"%s\"", url);
+
+		priv->cached += 1;
+		entry->timestamp = now;
+		return g_object_ref (entry->pixbuf);
+	}
+	else if (entry != NULL && entry->load_failed)
+	{
+		/* No need to try again */
+		priv->cached += 1;
+		return NULL;
+	}
+
+	if (entry == NULL)
 	{
 		char *filename;
 
@@ -499,13 +634,21 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 					&value);
 		g_value_unset (&value);
 
-		ephy_node_add_child (cache->priv->icons, icon);
+		/* This will also add an entry to the icons hash */
+		ephy_node_add_child (priv->icons, icon);
 
 		ephy_favicon_cache_download (cache, url, filename);
 
 		g_free (filename);
+
+		entry = g_hash_table_lookup (priv->icons_hash, url);
 	}
 
+	g_return_val_if_fail (entry != NULL, NULL);
+	g_return_val_if_fail (entry->pixbuf == NULL, NULL);
+
+	icon = entry->node;
+
 	/* update timestamp */
 	g_value_init (&value, G_TYPE_INT);
 	g_value_set_int (&value, now);
@@ -663,6 +806,7 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 
 	if (pixbuf == NULL)
 	{
+		entry->load_failed = TRUE;
 		return NULL;
 	}
 
@@ -675,5 +819,37 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 		pixbuf = scaled;
 	}
 
+	/* Put the icon in the cache */
+	LOG ("Putting pixbuf for \"%s\" in cache", url);
+
+	entry->pixbuf = g_object_ref (pixbuf);
+	entry->timestamp = now;
+	entry->load_failed = FALSE;
+
 	return pixbuf;
 }
+
+/**
+ * ephy_favicons_cache_clear:
+ * @cache:
+ * 
+ * Clears the favicon cache and removes any stored icon files from disk.
+ */
+void
+ephy_favicon_cache_clear (EphyFaviconCache *cache)
+{
+	EphyFaviconCachePrivate *priv = cache->priv;
+
+	g_return_if_fail (EPHY_IS_FAVICON_CACHE (cache));
+
+	remove_obsolete_icons (cache, NULL);
+	ephy_favicon_cache_save (cache);
+
+	/* Now remove any remaining files from the cache directory */
+	gnome_vfs_directory_visit (priv->directory,
+				   GNOME_VFS_FILE_INFO_DEFAULT,
+				   GNOME_VFS_DIRECTORY_VISIT_SAMEFS |
+				   GNOME_VFS_DIRECTORY_VISIT_LOOPCHECK,
+				   (GnomeVFSDirectoryVisitFunc) delete_file,
+				   cache);
+}
