@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyBrowser.cpp,v 1.91 2005/07/12 20:47:21 chpe Exp $
+ *  $Id: EphyBrowser.cpp,v 1.93 2005/08/06 21:11:24 crispin Exp $
  */
 
 #include "mozilla-config.h"
@@ -536,6 +536,9 @@ nsresult EphyBrowser::Init (GtkMozEmbed *mozembed)
 					 getter_AddRefs(mWebBrowser));
 	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
 
+	mWebBrowserFocus = do_QueryInterface (mWebBrowser);
+	NS_ENSURE_TRUE (mWebBrowserFocus, NS_ERROR_FAILURE);
+
 	mWebBrowser->GetContentDOMWindow (getter_AddRefs (mDOMWindow));
 	NS_ENSURE_TRUE (mDOMWindow, NS_ERROR_FAILURE);
 
@@ -658,8 +661,6 @@ EphyBrowser::AttachListeners(void)
 					     mPopupBlockEventListener, PR_FALSE);
 	rv |= mEventTarget->AddEventListener(nsEmbedString(kDOMWillOpenModalDialog),
 					     mModalAlertListener, PR_TRUE);
-	rv |= mEventTarget->AddEventListener(nsEmbedString(kDOMMouseScroll),
-					     mDOMScrollEventListener, PR_TRUE);
 	rv |= mEventTarget->AddEventListener(nsEmbedString(kDOMModalDialogClosed),
 					     mModalAlertListener, PR_TRUE);
 	rv |= mEventTarget->AddEventListener(nsEmbedString(kContextMenu),
@@ -861,13 +862,9 @@ nsresult EphyBrowser::GetTargetDocument (nsIDOMDocument **aDOMDocument)
 	}
 
 	/* Use the focused document */
-	nsCOMPtr<nsIWebBrowserFocus> webBrowserFocus;
-	webBrowserFocus = do_QueryInterface (mWebBrowser);
-	NS_ENSURE_TRUE (webBrowserFocus, NS_ERROR_FAILURE);
-
 	nsresult rv;
 	nsCOMPtr<nsIDOMWindow> DOMWindow;
-	rv = webBrowserFocus->GetFocusedWindow (getter_AddRefs(DOMWindow));
+	rv = mWebBrowserFocus->GetFocusedWindow (getter_AddRefs(DOMWindow));
 	if (NS_SUCCEEDED (rv) && DOMWindow)
 	{
 		return DOMWindow->GetDocument (aDOMDocument);
@@ -1324,3 +1321,21 @@ EphyBrowser::GetDocumentType ()
 
   return type;
 }
+
+#ifdef GTKMOZEMBED_BROKEN_FOCUS
+nsresult
+EphyBrowser::FocusActivate ()
+{
+	NS_ENSURE_STATE (mWebBrowserFocus);
+
+	return mWebBrowserFocus->Activate();
+}
+
+nsresult
+EphyBrowser::FocusDeactivate ()
+{
+	NS_ENSURE_STATE (mWebBrowserFocus);
+
+	return mWebBrowserFocus->Deactivate();
+}
+#endif /* GTKMOZEMBED_BROKEN_FOCUS */
