@@ -38,7 +38,7 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: MozDownload.cpp,v 1.65.2.6 2005/07/07 13:10:58 chpe Exp $
+ * $Id: MozDownload.cpp,v 1.73 2005/07/07 13:10:25 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -56,6 +56,8 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <glib/gi18n.h>
 
+#include <nsIIOService.h>
+#include <nsIURI.h>
 #include <nsIDOMDocument.h>
 #include <nsILocalFile.h>
 #include <nsIWebBrowserPersist.h>
@@ -755,7 +757,7 @@ static const char*
 file_is_compressed (const char *filename)
 {
         int i;
-        const char *compression[] = {".gz", ".bz2", ".Z", ".lz", NULL};
+        static const char * const compression[] = {".gz", ".bz2", ".Z", ".lz", NULL};
 
         for (i = 0; compression[i] != NULL; i++)
         {
@@ -777,7 +779,7 @@ parse_extension (const char *filename)
         if (compression != NULL)
         {
                 int i;
-                const char *extensions[] = {"tar", "ps", "xcf", "dvi", "txt", "text", NULL};
+                static const char * const extensions[] = {"tar", "ps", "xcf", "dvi", "txt", "text", NULL};
 
                 for (i = 0; extensions[i] != NULL; i++)
                 {
