@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id$
+ *  $Id: ephy-topic-action.c,v 1.73 2005/07/27 19:12:13 chpe Exp $
  */
 
 #include "config.h"
@@ -161,15 +161,24 @@ menu_deactivate_cb (GtkMenuShell *ms, GtkWidget *button)
 static void
 menu_activate_cb (GtkWidget *item, GtkAction *action)
 {
+	EphyBookmarks *bookmarks;
 	EphyNode *node;
 	const char *location;
+	char *address;
 
 	node = g_object_get_data (G_OBJECT (item), "node");
 	location = ephy_node_get_property_string
 		(node, EPHY_NODE_BMK_PROP_LOCATION);
+	g_return_if_fail (location != NULL);
 
-	ephy_link_open (EPHY_LINK (action), location, NULL,
+	bookmarks = ephy_shell_get_bookmarks (ephy_shell_get_default ());
+	address = ephy_bookmarks_resolve_address (bookmarks, location, NULL);
+	g_return_if_fail (address != NULL);
+
+	ephy_link_open (EPHY_LINK (action), address, NULL,
 			ephy_gui_is_middle_click () ? EPHY_LINK_NEW_TAB : 0);
+
+	g_free (address);
 }
 
 static void
@@ -188,14 +197,21 @@ ephy_topic_action_sync_label (GtkAction *gaction,
 	if (GTK_IS_TOOL_ITEM (proxy))
 	{
 		GtkWidget *label = NULL;
-		char *title;
+		char *title, *separator;
 
 		label = g_object_get_data (G_OBJECT (proxy), "label");
 		g_return_if_fail (label != NULL);
 
 		g_object_get (G_OBJECT (action), "label", &title, NULL);
+		g_return_if_fail (label != NULL);
 
-		gtk_label_set_label (GTK_LABEL (label), title);
+		/* In case this is a multi-hierarchy topic, we only want to
+		 * display the leaf name. See bug #310963.
+		 */
+		separator = g_strrstr (title, BOOKMARKS_HIERARCHY_SEP);
+
+		gtk_label_set_label (GTK_LABEL (label),
+				     separator != NULL ? separator + 2 : title);
 
 		g_free (title);
 	}
@@ -332,6 +348,7 @@ append_bookmarks_menu (EphyTopicAction *action, GtkWidget *menu, EphyNode *node,
 static void
 open_in_tabs_activate_cb (GtkWidget *item, EphyTopicAction *action)
 {
+	EphyBookmarks *bookmarks;
 	EphyNode *node;
 	GPtrArray *children;
 	EphyTab *tab = NULL;
@@ -350,16 +367,24 @@ open_in_tabs_activate_cb (GtkWidget *item, EphyTopicAction *action)
 
 	node_list = g_list_sort (node_list, (GCompareFunc) sort_bookmarks);
 
+	bookmarks = ephy_shell_get_bookmarks (ephy_shell_get_default ());
+
 	for (l = node_list; l != NULL; l = l->next)
 	{
 		EphyNode *child = (EphyNode *) l->data;
-		const char *address;
+		const char *location;
+		char *address;
 
-		address = ephy_node_get_property_string
+		location = ephy_node_get_property_string
 			(child, EPHY_NODE_BMK_PROP_LOCATION);
+		g_return_if_fail (location != NULL);
+
+		address = ephy_bookmarks_resolve_address (bookmarks, location, NULL);
+		g_return_if_fail (address != NULL);
 
 		tab = ephy_link_open (EPHY_LINK (action), address, tab,
 				      tab ? EPHY_LINK_NEW_TAB : EPHY_LINK_NEW_WINDOW);
+		g_free (address);
 	}
 
 	g_list_free (node_list);
