@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  $Id: ephy-find-toolbar.c,v 1.9 2005/06/04 20:16:24 chpe Exp $
+ *  $Id: ephy-find-toolbar.c,v 1.12 2005/07/29 20:40:38 crispin Exp $
  */
 
 #include "config.h"
@@ -249,13 +249,21 @@ entry_changed_cb (GtkEntry *entry,
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 	const char *text;
-	gboolean found = TRUE;
+	char *lowercase;
+	gboolean found = TRUE, case_sensitive;
 
 	text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
+
+	/* Search case-sensitively iff the string includes 
+	 * non-lowercase character.
+	 */
+	lowercase = g_utf8_strdown (text, -1);
+	case_sensitive = g_utf8_collate (text, lowercase) != 0;
+	g_free (lowercase);
+
+	ephy_embed_find_set_properties (get_find (toolbar), text, case_sensitive);
 #ifdef HAVE_TYPEAHEADFIND
 	found = ephy_embed_find_find (get_find (toolbar), text, priv->links_only);
-#else
-	ephy_embed_find_set_properties (get_find (toolbar), text, FALSE);
 #endif
 	set_controls (toolbar, found, found);
 }
@@ -368,7 +376,7 @@ ephy_find_toolbar_grab_focus (GtkWidget *widget)
 	EphyFindToolbar *toolbar = EPHY_FIND_TOOLBAR (widget);
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
-	gtk_widget_grab_focus (priv->entry);
+	gtk_widget_grab_focus (GTK_WIDGET (priv->entry));
 	g_return_if_fail (priv->explicit_focus);
 }
 
@@ -663,10 +671,13 @@ ephy_find_toolbar_open (EphyFindToolbar *toolbar,
 	}
 
 	gtk_widget_show (GTK_WIDGET (toolbar));
-	ephy_embed_activate (priv->embed);
 
 #ifdef HAVE_TYPEAHEADFIND
+	gtk_widget_grab_focus (GTK_WIDGET (priv->embed));
+
 	send_focus_change (priv->entry, TRUE);
+#else
+	gtk_widget_grab_focus (GTK_WIDGET (toolbar));
 #endif
 }
 
@@ -681,5 +692,7 @@ ephy_find_toolbar_close (EphyFindToolbar *toolbar)
 
 	/* first unset explicit_focus, else we get infinite recursion */
 	priv->explicit_focus = FALSE;
-	ephy_embed_activate (priv->embed);
+#ifndef HAVE_TYPEAHEADFIND
+	gtk_widget_grab_focus (GTK_WIDGET (priv->embed));
+#endif
 }
