@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.h,v 1.26 2005/07/24 12:26:22 chpe Exp $
+ *  $Id: ephy-tab.h,v 1.27 2005/08/01 15:57:48 chpe Exp $
  */
 
 #ifndef EPHY_TAB_H
@@ -24,6 +24,7 @@
 
 #include "ephy-embed.h"
 
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtkbin.h>
 
 G_BEGIN_DECLS
@@ -69,7 +70,9 @@ EphyTab		       *ephy_tab_for_embed		(EphyEmbed *embed);
 
 EphyEmbedDocumentType	ephy_tab_get_document_type	(EphyTab *tab);
 
-const char             *ephy_tab_get_icon_address	(EphyTab *tab);
+GdkPixbuf	       *ephy_tab_get_icon		(EphyTab *tab);
+
+const char	       *ephy_tab_get_icon_address	(EphyTab *tab);
 
 gboolean		ephy_tab_get_load_status	(EphyTab *tab);
 
